@@ -601,6 +601,34 @@ field<oT>::operator() (const uword in_row, const uword in_col, const uword in_sl
 
 
 
+#if defined(__cpp_multidimensional_subscript)
+  
+  //! element accessor; no bounds check
+  template<typename oT>
+  arma_inline
+  arma_warn_unused
+  oT&
+  field<oT>::operator[] (const uword in_row, const uword in_col)
+    {
+    return (*mem[in_row + in_col*n_rows]);
+    }
+  
+  
+  
+  //! element accessor; no bounds check
+  template<typename oT>
+  arma_inline
+  arma_warn_unused
+  const oT&
+  field<oT>::operator[] (const uword in_row, const uword in_col) const
+    {
+    return (*mem[in_row + in_col*n_rows]);
+    }
+  
+#endif
+
+
+
 //! element accessor; no bounds check
 template<typename oT>
 arma_inline
@@ -625,6 +653,34 @@ field<oT>::at(const uword in_row, const uword in_col) const
 
 
 
+#if defined(__cpp_multidimensional_subscript)
+  
+  //! element accessor; no bounds check
+  template<typename oT>
+  arma_inline
+  arma_warn_unused
+  oT&
+  field<oT>::operator[] (const uword in_row, const uword in_col, const uword in_slice)
+    {
+    return (*mem[in_row + in_col*n_rows + in_slice*(n_rows*n_cols)]);
+    }
+  
+  
+  
+  //! element accessor; no bounds check
+  template<typename oT>
+  arma_inline
+  arma_warn_unused
+  const oT&
+  field<oT>::operator[] (const uword in_row, const uword in_col, const uword in_slice) const
+    {
+    return (*mem[in_row + in_col*n_rows + in_slice*(n_rows*n_cols)]);
+    }
+  
+#endif
+
+
+
 //! element accessor; no bounds check
 template<typename oT>
 arma_inline
@@ -702,7 +758,7 @@ field<oT>::back() const
 
 
 template<typename oT>
-arma_cold
+arma_deprecated
 inline
 field_injector< field<oT> >
 field<oT>::operator<<(const oT& val)
@@ -713,7 +769,7 @@ field<oT>::operator<<(const oT& val)
 
 
 template<typename oT>
-arma_cold
+arma_deprecated
 inline
 field_injector< field<oT> >
 field<oT>::operator<<(const injector_end_of_row<>& x)
@@ -2960,7 +3016,7 @@ field_aux::load(field< std::string >& x, std::istream& is, const file_type type,
 
 
 
-#ifdef ARMA_EXTRA_FIELD_MEAT
+#if defined(ARMA_EXTRA_FIELD_MEAT)
   #include ARMA_INCFILE_WRAP(ARMA_EXTRA_FIELD_MEAT)
 #endif
 
